
$lib "LCD-EPSON1.02.LBX"                                    'Customized Epson library to work with Spark Fun breakout board

$regfile = "m168def.dat"                                    'ATMega 8, change if using different processors
$crystal = 8000000                                          '8 MHz

$hwstack = 256
$swstack = 128
$framesize = 128

'Set port D pins  as inputs except 7 which drive button LED
Ddrd = &B10000000
'Turn on internal pull-up resistors for port D pins, turn on LED
Portd = &B01111111

'Define Nokia LCD graphics display
Config Graphlcd = Color , Controlport = Portb , Cs = 1 , Rs = 0 , Scl = 3 , Sda = 2

'Color contants
Const Aquamarine = &B00111110
Const Black = &B00000000
Const Blue = &B00000011
Const Brown = &B01100000
Const Cyan = &B00011111
Const Darkblue = &B00000010
Const Darkgreen = &B00001100
Const Darkred = &B10100000
Const Green = &B00011100
Const Greenyellow = &B10011100
Const Grey = &B01101101
Const Magenta = &B11100011
Const Orange = &B11101100
Const Purple = &B01000010
Const Red = &B11100000
Const Silver = &B10010010
Const White = &B11111111
Const Yellow = &B11111100

'Sub to display text rotated 90 degrees
Declare Sub Lcdtext90(byval S As String , Byval Xoffset As Byte , Byval Yoffset As Byte , Byval Forecolor As Byte , Byval Backcolor As Byte , Byval Size As Byte)

'Function to convert resistance to temperature for Koolance sensor
Declare Function Gettemperature(byval Wres As Word) As Byte

'Set up ADC to read temperature sensor
Config Adc = Single , Prescaler = Auto , Reference = Avcc

Dim B As Byte
Dim Tmin As Byte
Tmin = 120
Dim Tmax As Byte
Dim W As Word
Dim S As Single
Dim Text As String * 16
Dim Tempc As Byte
Dim T(100) As Byte
Dim X As Byte
Dim Y As Byte

'Set up timer0 to count pump liters per minute
Config Timer0 = Counter , Edge = Rising

'Set up timer to track time
Config Timer1 = Timer , Prescale = 256
Stop Timer1

'Preload Timer Constant For 1 Second Duration At 8 Mhz for 256 prescale
Const Timer1pre = 34286
Timer1 = Timer1pre

'Handle time overflow (occurs every second)
On Timer1 Pulse

Cls
'Background for LCD display, driver requires 1 box prior to use
Box(0 , 0) -(130 , 130) , Blue
Waitms 50

Start Timer0
Start Timer1
Enable Interrupts
Enable Timer0
Enable Timer1
Enable Adc

'Draw y-axis
Line(26 , 26) -(110 , 26) , White
'Draw ticks
For B = 26 To 110 Step 14
   Line(b , 24) -(b , 28) , White
Next B
'Draw temperature labels
Lcdtext90 " 20" , 44 , 0 , White , Darkblue , 8
Lcdtext90 " 40" , 58 , 0 , White , Darkblue , 8
Lcdtext90 " 60" , 72 , 0 , White , Darkblue , 8
Lcdtext90 " 80" , 86 , 0 , White , Darkblue , 8
Lcdtext90 "100" , 100 , 0 , White , Darkblue , 8

Do

Loop

End



Sub Lcdtext90(byval S As String , Xoffset As Byte , Yoffset As Byte , Forecolor As Byte , Backcolor As Byte , Size As Byte)
   Local Tempstring As String * 1
   Local Temp As Byte
   Local A As Byte
   Local Pixels As Byte
   Local Charcount As Byte
   Local Rows As Byte
   Local Byteseach As Byte
   Local Blocksize As Byte
   Local Columns As Byte
   Local Columncount As Byte
   Local Rowcount As Byte
   Local Stringsize As Byte
   Local Xpos As Byte
   Local Ypos As Byte
   Local Pixel As Byte
   Local Pixelcount As Byte
   Local Fontaddress As Word
   Local Charaddress As Word

   If Size = 16 Then
      Fontaddress = Loadlabel(font16x16tt)                  'Load font address of 16x16 font
   Else
      Fontaddress = Loadlabel(font8x8tt)                    'Load font address of 8x8 font
   End If

   Rows = Cpeek(fontaddress)                                'Get rows
   Incr Fontaddress
   Byteseach = Cpeek(fontaddress)                           'Get bytesearch
   Incr Fontaddress
   Blocksize = Cpeek(fontaddress)                           'Get blocksize
   Incr Fontaddress
   Incr Fontaddress                                         'Skip rotation byte

   Columns = Blocksize / Rows                               'Calculate the numbers of columns
   Rows = Rows * 8                                          'Row is always 8 pixels high = 1 byte, so working with row in steps of 8.
   Rows = Rows - 1                                          'Want to start with row=0 instead of 1
   Columns = Columns - 1                                    'Same for the columns

   Stringsize = Len(s) - 1                                  'Size of the text string -1 because we must start with 0

   For Charcount = 0 To Stringsize                          'Loop through the numbers of characters that must be displayed
      Temp = Charcount + 1                                  'Cut the text string in seperate characters
      Tempstring = Mid(s , Temp , 1)
      Temp = Asc(tempstring) - 32                           'Font files start with character 32

      'Set character address
      Charaddress = Temp * Blocksize                        'Determine address offset of current character
      Charaddress = Charaddress + Fontaddress               'Determine character address
                                                            '90 degrees rotation
      For Rowcount = Rows To 0 Step -8                      'Loop is now counting down
         A = Rowcount + Xoffset
         A = A - 15                                         'Correction to set Xpos on Xoffset with rotation
         For Columncount = 0 To Columns
            Pixels = Cpeek(charaddress)
            Incr Charaddress
            Xpos = Columncount
            Temp = Charcount * Byteseach
            Xpos = Xpos + Temp
            Xpos = Xpos + Yoffset                           'We want that Xoffset is still Xoffset, so we need here the change from x to y
            For Pixelcount = 7 To 0 Step -1
               Ypos = A + Pixelcount
               Pixel = Pixels.0
               If Pixel = 1 Then
                  Pixel = Forecolor
               Else
                  Pixel = Backcolor
               End If
               Pset Ypos , Xpos , Pixel
               Shift Pixels , Right
            Next Pixelcount
         Next Columncount
      Next Rowcount
   Next Charcount
End Sub


Function Gettemperature(byval Wres As Word)
   'Range 19 - 119C
   Local Address As Word
   'Get Address of temperature resistance table
   Address = Loadlabel(kooltemp)

   Local Btemp As Byte
   Local Bcount As Byte
   Local Temperature As Byte
   Local Wmin As Word

   'Loop through table until temperature found
   For Bcount = 1 To 101
      'Get temperature
      Temperature = Cpeek(address)
      Incr Address
      'Get resistance high byte
      Btemp = Cpeek(address)
      Wmin = Btemp
      Shift Wmin , Left , 8
      Incr Address
      'Get resistance low byte
      Btemp = Cpeek(address)
      Wmin = Wmin Or Btemp
      'Check if within range
      If Wres > Wmin Then
         Exit For
      Else
         Incr Address
      End If
   Next Bcount

   Gettemperature = Temperature
End Function


Pulse:
   Stop Timer0
   Stop Timer1

   'Calculate liters per minute
   W = Counter0 * 241
   Text = Str(w)
   Text = Format(text , "00.000")
   Text = Left(text , 4)
   Text = Text + " LPM"
   Lcdtext90 Text , 7 , 0 , Cyan , Purple , 16

   'Get temperature sensor ADC value
   S = 0
   'Sample ADC multiple times to improve accuracy
   For B = 1 To 64
      W = Getadc(0)
      S = S + W
      Waitms 1
   Next B
   Shift S , Right , 6                                      'Get average
   'Convert ADC to resistance using Ohm's law
   'Multiply by Vin = 3.3V (S = S * 3.3)
   'then divide by ADC range 1024 to get Vout (S = S / 1024)
   S = S * 0.00322265625                                    'Multiply by coefficient based on above calculations
   S = 33000 / S                                            'Vin times fixed resistor divided by Vout
   W = S - 10000                                            'Subtract fixed resistor to get unknown resistance

   'Translate resistance to temperature in Celcius
   Tempc = Gettemperature(w)
   Text = Str(tempc)
   Text = Format(text , "000")
   Text = Text + Chr(127)                                   'Font 16x16 file modifed to add degree symbol at 127
   Text = Text + "C"
   If Tempc < 50 Then
      'Temperature Cool
      Lcdtext90 Text , 121 , 28 , Black , Green , 16
   Elseif Tempc < 70 Then
      'Temperature Warm
      Lcdtext90 Text , 121 , 28 , Black , Yellow , 16
   Else
      'Temperature Hot
      Lcdtext90 Text , 121 , 28 , White , Red , 16
   End If

   'Track minimum and maximum temperatures
   If Tempc < Tmin Then Tmin = Tempc
   If Tempc > Tmax Then Tmax = Tempc
   Text = Str(tmax)
   Text = Format(text , "000")
   Text = "-" + Text
   Text = Str(tmin) + Text
   Text = Format(text , "000-000")
   Text = "RANGE: " + Text
   Text = Text + Chr(248)                                   'Degree symbol
   Text = Text + "C"
   Lcdtext90 Text , 24 , 0 , Black , Silver , 8

   'Draw chart
   For B = 1 To 100
      If B = 100 Then
        T(b) = Tempc
      Else
        T(b) = T(b + 1)
      End If
      'Determine x position
      X = B + 28
      'Determine y position
      S = T(b) * .7
      Y = S + 26
      'Draw temperature
      If T(b) < 50 Then
         'Temperature Cool
         Line(26 , X) -(y , X) , Green
      Elseif T(b) < 70 Then
         'Temperature Warm
         Line(26 , X) -(y , X) , Yellow
      Else
         'Temperature Hot
         Line(26 , X) -(y , X) , Red
      End If
      'Line(26 , X) -(y , X) , Cyan
      'Clear chart above temperature
      Y = Y + 1
      Line(y , X) -(110 , X) , Black
   Next B

   'Reset counters
   Counter0 = 0
   Timer1 = Timer1pre
   Start Timer0
   Start Timer1
   Return


$include "font8x8TT.font"
$include "font16x16TT.font"
$include "KoolTemp.bas"